function [M1,pol,g] = espira1(y)

%  Input: y - vector of sumples
%         
% Output: g, pol  vector of coefficients and poles in the exponential sum 
%         M1 length of the exponential sum


tol = 1e-13; %default tolerance for AAA algorithm 
F=fft(y);     % compute DFT
M = length(F); % number of modified DFT values
Z=0:M-1; Z=exp(2*pi*1i.*Z/M);  %set knots on the unit circle
Z=Z(:); F=F(:); % work with column vectors
F=F.*Z.^(-1); % modify the DFT values
mmax = floor(M/2)-1;
if nargin<2, tol = 1e-13; end % default relative tol 1e-13

% We further use AAA algorithm developed in 
%Y. Nakatsukasa, O. Sete, and L.N. Trefethen. The AAA algorithm for rational approximation. SIAM J. Sci. Comput., 40(3):A1494–A1522, 2018.
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
SF = spdiags(F,0,M,M); % diagonal matrix with values of F
J = 1:M; z = []; f = []; J=J(:); 
C = []; errvec = []; R=zeros(M,1); R=R(:); % initializations

% find the coefficients with largest absolute value
[R1,k1] = max(abs(F-R)); j1=find(J==k1);
R = R+ R1;                % initial rational approximation
z=[z;Z(j1)];              %update sets
f=[f;F(j1)];
J(J==j1)=[];              % update index set
C=[C 1./(Z-Z(j1))];       % first column of Cauchy matrix
err = norm(F(J)-R(J),inf); % max error at sample points
errvec = [errvec; err]; 
if err <= tol*norm(F,inf), end % stop if converged

for m = 2:mmax   % main loop
[~,j] = max(abs(F(J)-R(J)));  % select next support point
z = [z; Z(J(j))];
f = [f; F(J(j))]; % update support points, data values
C = [C 1./(Z-Z(J(j)))];  % next column of Cauchy matrix
J(j) = []; % update index vector
Sf = diag(f); % right scaling matrix
A=SF*C-C*Sf; % Loewner matrix
A=A(J,:);
[~,G,V] = svd(A); % SVD of A
w = V(:,m);  % last singular vector of A
N = C*(w.*f); D = C*w; % numerator and denominator
R = F;
R(J) = N(J)./D(J); % rational approximation
err = norm(F(J)-R(J),inf);
errvec = [errvec; err]; % max error at sample points
if err <= tol*norm(F,inf), break, end % stop if converged
end
% end of AAA algorithm
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

j2=find(abs(w)<=1e-13); %find possible zero components in w
w(j2)=[];              %remove zero components in w
z(j2)=[];              % update corresponding interpolation points
f(j2)=[];              % update corresponding values

[M1,g,AB,pol] = prz(F,Z,z,f,w,M); % poles and zeros and partial decomposition


function [M1,g,AB,pol] = prz(F,Z,z,f,w,M) 
% compute poles pol of rational function (wanted parameters)
% AB complex nominator vector in partial fraction decomposition
% g coefficients
m = length(w); 
M1=m; % length of the exponential sum
% computation of poles of the rational function
B = eye(m+1);
B(1,1) = 0;
E = [0 w.'; ones(m,1) diag(z)];
pol = eig(E,B);
pol = pol(~isinf(pol)); %  poles

% computation of coefficients g 
CC=(-1)./bsxfun(@minus,z, pol.');
AB=linsolve(CC,f);
j1=find(abs(AB)<=1e-7);  % find the zero components of numerator
AB(j1)=[];
pol(j1)=[];
%a=log(pol);
g=-AB./(1-pol.^(M));

disp('condition number of the Cauchy matrix for ESPIRA I');
conditionC=cond(CC);
disp(conditionC);

 % computation of the periodic part
 Z1 = setdiff(Z,z,'stable');
 F1= setdiff(F,f,'stable');
 CC = 1./bsxfun(@minus,Z1,z.'); %Cauchy matrix
 r = (CC*(w.*f))./(CC*w); %compute rational function at Z
 f2=F1- r; %compute the difference between Fourier coefficientvector and rational function  
 ii = find(abs(f2)>0.000001); % find indices belonging to periodic part
 if ~isempty(ii)
     r=length(ii);
     for k=1:r
 n=find(Z==Z1(ii(k)));
 a=2*pi*1i*(n-1)./M;
 %a=[a;2*pi*1i*(n-1)./M];
 pol=[pol;exp(a(end))];
 g=[g;f2(ii(k))*exp(2*pi*1i*(n-1)./M)./M];
     end
 end
 

